/*
 * Copyright (c) 1995, 1996, 1997 The University of Utah and
 * the Computer Systems Laboratory at the University of Utah (CSL).
 *
 * This file is part of Flick, the Flexible IDL Compiler Kit.
 *
 * Flick is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * Flick is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Flick; see the file COPYING.  If not, write to
 * the Free Software Foundation, 59 Temple Place #330, Boston, MA 02111, USA.
 */

#include <assert.h>

#include <mom/compiler.h>
#include <mom/c/libcast.h>

#include "mach3.h"

static void do_stub(pres_c_1 *pres, cast_ref c_func,
		    mint_ref request_itype, mint_ref reply_itype,
		    pres_c_inline request_i, pres_c_inline reply_i,
		    pres_c_inline target_i,
		    pres_c_stub_op_flags op_flags)
{
	assert(pres);
	
	cast_def *cfunc = &pres->cast.cast_scope_val[c_func];
	cast_func_type *cfunct = &cfunc->u.cast_def_u_u.func_type;

	mint_ref            operation_itype;
	pres_c_inline       operation_inline;
	
	int assumptions = RPCM_TRUSTED; /* XXX */
	char *emergency_return_string;
	
	/*****/
	
	if (cfunct->return_type->kind != CAST_TYPE_VOID)
		emergency_return_string = "return KERN_FAILURE";
	else
		emergency_return_string = "return";
	emergency_return_value = cast_new_expr_name(emergency_return_string);
	
        /* Build the parameter marshal/deallocation code. */
	mach3_mu_state must_in(pres,
			       (MUST_ENCODE | MUST_DEALLOCATE), assumptions,
			       "client");
	
	remove_idl_and_interface_ids(pres,
				     request_itype, request_i,
				     &operation_itype, &operation_inline);
	must_in.set_id_expected(operation_itype);
	
	must_in.mu_func_params(c_func, operation_itype, operation_inline);
	must_in.mu_end();
	
	/* Build the unmarshal code, but only if this is a two-way stub. */
	mach3_mu_state must_out(pres,
				(MUST_DECODE | MUST_ALLOCATE), assumptions,
				"client");
	if (reply_i) {
		remove_idl_and_interface_ids(pres,
					     reply_itype, reply_i,
					     &operation_itype,
					     &operation_inline);
		/*
		 * Do not call `remove_operation_id' to remove the operation
		 * identifier.  MIG transmits operation reply codes, so we do,
		 * too.
		 */
		must_out.set_id_expected(operation_itype);
		must_out.mu_func_params(c_func,
					operation_itype, operation_inline);
		must_out.mu_end();
	} else {
		assert(must_out.max_msg_size == 0);
	}
	
	/* Build the target object marshaling code. */
	mach3_target_mu_state must_target(pres,
					  MUST_ENCODE, 0,
					  "client");
	must_target.mu_func_params(c_func, must_target.pres->mint.
				   standard_refs.interface_name_ref, target_i);
	must_target.mu_end();
	
	/* Start writing the client stub.  */
	cast_w_func_type(cfunc->name, cfunct, 0);
	w_printf("\n{\n");
	w_i_printf(1, "FLICK_BUFFER _temp;\n");
	w_i_printf(1, "FLICK_BUFFER *_stream;\n");
	w_i_printf(1, "mach_msg_return_t _rc;\n");
	
	if (cfunct->return_type->kind != CAST_TYPE_VOID) {
		/*
		 * Initialize `_return' to be zero.  This eliminates compiler
		 * warnings about returning uninitialized data.
		 */
		cast_expr init_expr
			= cast_new_expr_assign(
				cast_new_expr_name("_return"),
				cast_new_expr_name("KERN_SUCCESS")
				);
		
		w_indent(1);
		cast_w_type("_return", cfunct->return_type, 1);
		w_printf(";\n");
		
		w_printf("\n");
		cast_w_stmt(cast_new_stmt_expr(init_expr), 1);
	} else
		w_printf("\n");
	
	w_i_printf(1,
		   "flick_%s_client_start_encode();\n",
		   must_in.get_be_name());
	cast_w_stmt(must_target.c_block, 1);
	
	/* Output the general marshaling code. */
	cast_w_stmt(must_in.c_block, 1);
	w_i_printf(1,
		   "flick_%s_client_end_encode();\n",
		   must_in.get_be_name());

	w_i_printf(1, "_stream->msg_buf->Head.msgh_bits = MACH_MSGH_BITS(19, MACH_MSG_TYPE_MAKE_SEND_ONCE);\n");
	if (must_in.is_complex)
		w_i_printf(1,
			   "_stream->msg_buf->Head.msgh_bits |= "
			   "MACH_MSGH_BITS_COMPLEX;\n");
	
	/* Invoke the appropriate runtime function to perform the RPC. */
	w_i_printf(1,
		   "_rc = flick_%s_%s(_stream);\n",
		   must_in.get_be_name(),
		   ((op_flags & PRES_C_STUB_OP_FLAG_ONEWAY) ? "send" : "rpc")
		);
	
	/* Check for errors from the IPC operation. */
	w_i_printf(1,
		   "if (_rc != MACH_MSG_SUCCESS)\n");
	w_i_printf(2,
		   "flick_%s_decode_client_error"
		   "(%s, FLICK_ERROR_COMMUNICATION, %s, %s);\n",
		   must_in.pres->pres_context,
		   emergency_return_string,
		   must_in.get_encode_name(),
		   must_in.get_be_name());
	
	/* Output the reply unmarshaling code. */
	if (op_flags & PRES_C_STUB_OP_FLAG_ONEWAY) {
		w_printf("}\n\n");
		return;
	}
	
	if (reply_i) {
		w_i_printf(1,
			   "flick_%s_client_start_decode();\n",
			   must_in.get_be_name());
		cast_w_stmt(must_out.c_block, 1);
		w_i_printf(1,
			   "flick_%s_client_end_decode();\n",
			   must_in.get_be_name());
	}
		
	if (cfunct->return_type->kind != CAST_TYPE_VOID)
		w_i_printf(1, "return _return;\n");
	w_printf("}\n\n");
}

/* Start the client stub */
void w_client_stub(pres_c_1 *pres, int stub_idx)
{
	assert(pres);

	pres_c_stub *stub = &pres->stubs.stubs_val[stub_idx];
	assert(stub->kind == PRES_C_CLIENT_STUB);
	pres_c_client_stub *cstub = &stub->pres_c_stub_u.cstub;

	assert(cstub->request_i);
	assert(cstub->reply_i);

	do_stub(pres, cstub->c_func,
		cstub->request_itype, cstub->reply_itype,
		cstub->request_i, cstub->reply_i, cstub->target_i,
		cstub->op_flags);
}

void w_send_stub(pres_c_1 *pres, int stub_idx)
{
	assert(pres);

	pres_c_stub *stub = &pres->stubs.stubs_val[stub_idx];
	assert(stub->kind == PRES_C_SEND_STUB);
	pres_c_send_stub *sstub = &stub->pres_c_stub_u.send_stub;

	do_stub(pres, sstub->c_func,
		sstub->msg_itype, mint_ref_null,
		sstub->msg_i, 0, sstub->target_i,
		PRES_C_STUB_OP_FLAG_ONEWAY /* XXX --- Right? */);
}

/* End of file. */

