/*
 * Copyright (c) 1996, 1997 The University of Utah and
 * the Computer Systems Laboratory at the University of Utah (CSL).
 * All rights reserved.
 *
 * Permission to use, copy, modify and distribute this software is hereby
 * granted provided that (1) source code retains these copyright, permission,
 * and disclaimer notices, and (2) redistributions including binaries
 * reproduce the notices in supporting documentation, and (3) all advertising
 * materials mentioning features or use of this software display the following
 * acknowledgement: ``This product includes software developed by the
 * Computer Systems Laboratory at the University of Utah.''
 *
 * THE UNIVERSITY OF UTAH AND CSL ALLOW FREE USE OF THIS SOFTWARE IN ITS "AS
 * IS" CONDITION.  THE UNIVERSITY OF UTAH AND CSL DISCLAIM ANY LIABILITY OF
 * ANY KIND FOR ANY DAMAGES WHATSOEVER RESULTING FROM THE USE OF THIS SOFTWARE.
 *
 * CSL requests users of this software to return to csl-dist@cs.utah.edu any
 * improvements that they make and grant CSL redistribution rights.
 */

#include "iiop-link.h"

/* needed only for flick_check_byte_order */
#include <flick/encode/cdr.h>

CORBA_boolean CORBA_Object_is_nil(CORBA_Object ths)
{
	return !ths;
}

CORBA_Object CORBA_Object_duplicate(CORBA_Object ths)
{
	CORBA_Object res;
	
	if (!ths)
		return ths;
	res = t_malloc(struct flick_client, 1);

	res->boa = ths->boa;
	
	res->key._buffer = t_malloc(char, ths->key._length);
	res->key._maximum = 1024;
	res->key._length = ths->key._length;
	memcpy(res->key._buffer, ths->key._buffer, ths->key._length);
	
	res->type_id = t_malloc(char, (ths->type_id_len + 1));
	strncpy((char *) res->type_id, ths->type_id, (ths->type_id_len + 1));
	
	res->type_id_len = ths->type_id_len;
	
	return res;
}

void CORBA_Object_release(CORBA_Object ths)
{
	if (ths && ths->key._buffer)
		free(ths->key._buffer);
	if (ths && ths->type_id)
		free((char *) ths->type_id);
	if (ths)
		free(ths);
}

CORBA_unsigned_long CORBA_Object_hash(CORBA_Object ths,
				      CORBA_unsigned_long maximum)
{
	unsigned long hash = 0;
	int i;
	if (!ths)
		return hash;
	for (i = 0; i < ths->key._length; i++) {
		hash = hash + (unsigned long)ths->key._buffer[i];
		hash = hash << 8;
		hash = hash % maximum;
	}
	return hash;
}

CORBA_boolean CORBA_Object_is_equivalent(CORBA_Object ths,
					 CORBA_Object other_object)
{
	int i;
	if (ths == other_object)
		return 1;
	if (!ths || !other_object)
		return 0;
	if (ths->boa != other_object->boa)
		return 0;
	if (ths->key._length != other_object->key._length)
		return 0;
	for (i = 0; i < ths->key._length; i++)
		if (ths->key._buffer[i] != other_object->key._buffer[i])
			return 0;
	return 1;
}

void flick_object_initialize(CORBA_Object obj)
{
	CORBA_ORB orb = obj->boa->orb;
	CORBA_BOA boa = obj->boa;
	int x = 1;
	
	/*
	 * XXX --- It is complete crap to establish our ORB connection here,
	 * because we have no way to report failure to the client stub (i.e.,
	 * no way to return a CORBA exception).  We should establish our ORB
	 * connection in `flick_client_send_request' instead, and just allocate
	 * our message buffers here.
	 */
	
	if (!orb->ORBid) {
		struct hostent *he = gethostbyname(orb->hostname);
		
		if (!he) {
			fprintf(stderr,
				"Error: cannot locate host `%s'.\n",
				orb->hostname);
			exit(1);
		}
		
                flick_is_little_endian = *(char *)(&x);
		flick_check_byte_order();
		orb->ipaddr = t_calloc(struct sockaddr_in, 1);
		orb->ipaddr->sin_addr.s_addr = *(long *)*he->h_addr_list;
		orb->ipaddr->sin_family = AF_INET;
		orb->ipaddr->sin_port = htons(boa->hostport);
		orb->ORBid = t_malloc(char, 11);
		strcpy(orb->ORBid, "FlickORB--");
		orb->OA_count = 1;
		orb->boas = t_malloc(CORBA_BOA, 1);
		orb->boas[0] = boa;
		orbs = t_realloc(orbs, CORBA_ORB, ++orb_count);
		orbs[orb_count - 1] = orb;
	}
	boa->OAid = t_malloc(char, 11);
	strcpy(boa->OAid, "FlickBOA--");
	
	boa->socket_fd = socket(AF_INET, SOCK_STREAM, IPPROTO_TCP);
	x = connect(boa->socket_fd, (struct sockaddr *)orb->ipaddr, sizeof(*orb->ipaddr));
	
	if (!boa->in)
		boa->in = t_malloc(FLICK_BUFFER, 1);
	boa->in->real_buf_start = boa->in->real_buf_end
				 = boa->in->buf_read = 0;
	boa->in->buf_start = boa->in->buf_current = t_calloc(char, 8192);
	boa->in->buf_end = boa->in->buf_start + 8192;
	
	if (!boa->out)
		boa->out = t_malloc(FLICK_BUFFER, 1);
	boa->out->real_buf_start = boa->out->buf_read = boa->out->buf_start
				= boa->out->buf_current = t_calloc(char, 8192);
	boa->out->real_buf_end = boa->out->buf_end = boa->out->buf_start + 8192;
}

/* End of file. */

