/*
 * Copyright (c) 1995, 1996, 1997 The University of Utah and
 * the Computer Systems Laboratory at the University of Utah (CSL).
 * All rights reserved.
 *
 * Permission to use, copy, modify and distribute this software is hereby
 * granted provided that (1) source code retains these copyright, permission,
 * and disclaimer notices, and (2) redistributions including binaries
 * reproduce the notices in supporting documentation, and (3) all advertising
 * materials mentioning features or use of this software display the following
 * acknowledgement: ``This product includes software developed by the
 * Computer Systems Laboratory at the University of Utah.''
 *
 * THE UNIVERSITY OF UTAH AND CSL ALLOW FREE USE OF THIS SOFTWARE IN ITS "AS
 * IS" CONDITION.  THE UNIVERSITY OF UTAH AND CSL DISCLAIM ANY LIABILITY OF
 * ANY KIND FOR ANY DAMAGES WHATSOEVER RESULTING FROM THE USE OF THIS SOFTWARE.
 *
 * CSL requests users of this software to return to csl-dist@cs.utah.edu any
 * improvements that they make and grant CSL redistribution rights.
 */

/*
 * Client-side helper routines for the Flick Mach3 backend
 */

#include <stdio.h>
#include <stdlib.h>
#include <mach/mig_support.h>
#include <mach/mig_errors.h>
#include <mach/port.h>
#include <mach/message.h>
#include <flick/link/mach3mig.h>
#include <flick/encode/mach3mig.h>

void print_message(mig_reply_header_t *rep, vm_offset_t size);

int
flick_mach3mig_rpc_grow_buf(flick_marshal_stream rpc, int size)
{
  int buf_size = (int)rpc->msg_end - (int)rpc->msg_buf;
  int offset = (int)rpc->chunk_ptr - (int)rpc->msg_buf;
  int new_size = (buf_size + size + 3) & ~3;
  
#if 0
  printf("Growing buffer...");
#endif
  
  if (!(rpc->msg_buf 
	= (mig_reply_header_t *)nonposix_realloc(rpc->msg_buf, new_size))) {
	  return FLICK_NO_MEMORY;
  }
  rpc->msg_end = (void *)&(((char *)rpc->msg_buf)[new_size]);
  rpc->chunk_ptr = (void *)&(((char *)rpc->msg_buf)[offset]);
  
  __buffer = rpc->msg_buf;
  __buflen = new_size;
  
#if 0
  printf("done!\n");
#endif
  
  return MACH_MSG_SUCCESS;
}

mach_msg_return_t flick_mach3mig_rpc(flick_marshal_stream rpc)
{
  mach_msg_return_t mr;
  mach_port_t reply_port;
  
  int msg_size = (((int)rpc->chunk_ptr - (int)rpc->msg_buf) + 3) & ~3;
  int buf_size = (((int)rpc->msg_end - (int)rpc->msg_buf) + 3) & ~3;
  
  /* Grab a port to receive the reply on.  */
  
  reply_port = mig_get_reply_port();
  rpc->msg_buf->Head.msgh_local_port = reply_port;
  
  /*
   * Consider the following cases:
   *	1) Errors in pseudo-receive (eg, MACH_SEND_INTERRUPTED
   *	plus special bits).
   *	2) Use of MACH_SEND_INTERRUPT/MACH_RCV_INTERRUPT options.
   *	3) RPC calls with interruptions in one/both halves.
   *	4) Exception reply messages that are bigger
   *	   than the expected non-exception reply message.
   *
   * We refrain from passing the option bits that we implement
   * to the kernel.  This prevents their presence from inhibiting
   * the kernel's fast paths (when it checks the option value).
   */
  
  rpc->msg_buf->Head.msgh_size = msg_size;
#if 0
  printf("RPC-SND:\n");
  print_message(rpc->msg_buf, rpc->msg_buf->Head.msgh_size);
#endif
  mr = mach_msg(&rpc->msg_buf->Head, MACH_SEND_MSG|MACH_RCV_MSG|MACH_RCV_LARGE|MACH_MSG_OPTION_NONE,
		msg_size, buf_size, reply_port, MACH_MSG_TIMEOUT_NONE, MACH_PORT_NULL);
  if (mr != MACH_MSG_SUCCESS)
    {
	  
      while (mr == MACH_SEND_INTERRUPTED)
	{
	  /* Retry both the send and the receive.  */
	  mr = mach_msg_trap(&rpc->msg_buf->Head,
			     MACH_SEND_MSG|MACH_RCV_MSG|MACH_RCV_LARGE,
			     msg_size, buf_size, reply_port,
			     MACH_MSG_TIMEOUT_NONE, MACH_PORT_NULL);
	}
	  
      while ((mr == MACH_RCV_INTERRUPTED) || (mr == MACH_RCV_TOO_LARGE))
	{
	  if (mr == MACH_RCV_TOO_LARGE)
	    {
	      /* Oops, message too large - grow the buffer.  */
	      buf_size = rpc->msg_buf->Head.msgh_size;
	      if (!(rpc->msg_buf =
		    (mig_reply_header_t *)nonposix_realloc(rpc->msg_buf, buf_size)))
		{
		  mig_dealloc_reply_port(/*reply_port*/);
		  return FLICK_NO_MEMORY;
		}
	      __buffer = rpc->msg_buf;
	      __buflen = buf_size;
	      
	      /* JLS - I don't see why this is necessary... */
	      /*rpc->msg_buf->Head = *(mach_msg_header_t*)(rpc->msg_buf);*/
	    } 
	      
	      
	  /* Retry the receive only
	     (the request message has already been sent successfully).  */
	  mr = mach_msg_trap(&rpc->msg_buf->Head,
			     MACH_RCV_MSG|MACH_RCV_LARGE,
			     0, buf_size, reply_port, 0, 0);
	}
	  
      if (mr != MACH_MSG_SUCCESS)
	{
#if 0
	  if ((mr == MACH_SEND_INVALID_REPLY) ||
	      (mr == MACH_SEND_INVALID_MEMORY) ||
	      (mr == MACH_SEND_INVALID_RIGHT) ||
	      (mr == MACH_SEND_INVALID_TYPE) ||
	      (mr == MACH_SEND_MSG_TOO_SMALL) ||
	      (mr == MACH_RCV_INVALID_NAME))
	    mig_dealloc_reply_port(reply_port);
	  else
	    mig_put_reply_port(reply_port);
#endif
	  return mr;
	}
    }
	
#if 0
  printf("RPC-RCV: (%d, 0x%08x)\n", mr, mr);
  print_message(rpc->msg_buf, rpc->msg_buf->Head.msgh_size);
#endif

  /* Stash the reply port again for future use.  */
  /* mig_put_reply_port(reply_port); */
	
  if (rpc->msg_buf->RetCode != KERN_SUCCESS)
    fprintf(stderr,"Return error code: %x\n", rpc->msg_buf->RetCode);
	
  rpc->chunk_ptr = (&rpc->msg_buf) + sizeof(mig_reply_header_t);
  
  return MACH_MSG_SUCCESS;
}



mach_msg_return_t flick_mach3mig_send(flick_marshal_stream send)
{
  mach_msg_return_t mr;

  int msg_size = (((int)send->chunk_ptr - (int)send->msg_buf) + 3) & ~3;
	
  send->msg_buf->Head.msgh_local_port = MACH_PORT_NULL;

  send->msg_buf->Head.msgh_size = msg_size;
#if 0
  printf("Send:\n");
  print_message(send->msg_buf, send->msg_buf->Head.msgh_size);
#endif
  mr = mach_msg(&send->msg_buf->Head, MACH_SEND_MSG|MACH_RCV_MSG|MACH_RCV_LARGE|MACH_MSG_OPTION_NONE,
		msg_size, 0, MACH_PORT_NULL, 0, 0);
  
  if (mr != MACH_MSG_SUCCESS)
    {
	  
      while (mr == MACH_SEND_INTERRUPTED)
	{
	  /* Retry both the send and the receive.  */
	  mr = mach_msg_trap(&send->msg_buf->Head,
			     MACH_SEND_MSG,
			     msg_size, 0, MACH_PORT_NULL, 0, 0);
	}
	  
      if (mr != MACH_MSG_SUCCESS)
	{
#if 0
	if ((mr == MACH_SEND_INVALID_REPLY) ||
	      (mr == MACH_SEND_INVALID_MEMORY) ||
	      (mr == MACH_SEND_INVALID_RIGHT) ||
	      (mr == MACH_SEND_INVALID_TYPE) ||
	      (mr == MACH_SEND_MSG_TOO_SMALL) ||
	      (mr == MACH_RCV_INVALID_NAME))
	    mig_dealloc_reply_port(reply_port);
	  else
	    mig_put_reply_port(reply_port);
#endif
	return mr;
	}
    }
	
  if (send->msg_buf->RetCode != KERN_SUCCESS)
    fprintf(stderr,"Return error code: %x\n", send->msg_buf->RetCode);
	
  return MACH_MSG_SUCCESS;
}

/* End of file. */

