/*
 * Copyright (c) 1997 The University of Utah and
 * the Computer Systems Laboratory at the University of Utah (CSL).
 *
 * This file is part of Flick, the Flexible IDL Compiler Kit.
 *
 * Flick is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * Flick is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Flick; see the file COPYING.  If not, write to
 * the Free Software Foundation, 59 Temple Place #330, Boston, MA 02111, USA.
 */

#include <stdio.h>
#include <flick/link/iiop.h>
#include "oneway-server.h"

#define flick_copy_const_string(dest,src) {			\
	dest = (char *)CORBA_alloc(sizeof(char)*strlen(src)+1);	\
	strcpy(dest, src);					\
}

int register_objects(CORBA_ORB orb, CORBA_BOA boa, int argc, char **argv) 
{
	int i;
	CORBA_ReferenceData key;
	key._maximum = 1024;
	for (i = 1; i < argc; i++) {
		if (!strcmp(argv[i], "-I") && i + 1 < argc) {
			key._buffer = argv[++i];
			key._length = strlen(key._buffer);
			CORBA_BOA_obj_is_ready(boa,
					       "onewaytest",
					       &onewaytest_server,
					       &key);
		}
	}
	return 0;
}


CORBA_short onewaytest_shortlong(onewaytest _obj, CORBA_long *x, CORBA_Environment *_ev)
{
   printf("onewaytest_shortlong Received InOut %d.\n",*x);
   *x *= 3;
   return 3;
}

void onewaytest_voidlong(onewaytest _obj, CORBA_long x, CORBA_Environment *_ev)
{
	printf("onewaytest_voidlong Received In %d.\n",x);
}

void onewaytest_onewaylong(onewaytest _obj, CORBA_long x, CORBA_Environment *_ev)
{
	printf("onewaytest_onewaylong Oneway received In %d.\n",x);
}

void onewaytest_onewaystring(onewaytest _obj, CORBA_char *x, CORBA_Environment *_ev)
{
	printf("onewaytest_onewaystring Oneway string received: %s.\n",x);
}

CORBA_char *onewaytest_stringreturn(onewaytest _obj, CORBA_Environment *_ev)
{
	char *result;
	printf("onewaytest_stringreturn Sending Hello World.\n");

	flick_copy_const_string(result, "Hello World");
	return result;
}

CORBA_long onewaytest_longstring(onewaytest _obj, CORBA_char **x, CORBA_Environment *_ev)
{
	printf("onewaytest_longstring Received InOut %s: Onewaytest",*x);
	CORBA_free(*x);
        flick_copy_const_string(*x, "Fabulous!");
	return 21;
}

CORBA_char *onewaytest_stringlong(onewaytest _obj, CORBA_long *x, CORBA_Environment *_ev)
{
	char *result;
	*x = 957;
	printf("onewaytest_stringlong sending Out %d and Out 'My House Number'.\n",*x);
	flick_copy_const_string(result, "My House Number");
	return result;
}

/* End of file. */

