/*
 * Copyright (c) 1997 The University of Utah and
 * the Computer Systems Laboratory at the University of Utah (CSL).
 *
 * This file is part of Flick, the Flexible IDL Compiler Kit.
 *
 * Flick is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * Flick is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Flick; see the file COPYING.  If not, write to
 * the Free Software Foundation, 59 Temple Place #330, Boston, MA 02111, USA.
 */

#include <stdio.h>
#include <flick/link/iiop.h>
#include "test1-server.h"

int register_objects(CORBA_ORB orb, CORBA_BOA boa, int argc, char **argv) 
{
	int i;
	CORBA_ReferenceData key;
	key._maximum = 1024;
	for (i = 1; i < argc; i++) {
		if (!strcmp(argv[i], "-I") && i + 1 < argc) {
			key._buffer = argv[++i];
			key._length = strlen(key._buffer);
			CORBA_BOA_obj_is_ready(boa,
					       "test1",
					       &test1_server,
					       &key);
		}
	}
	return 0;
}



#define MKTST(name, type) type test1_test_##name(test1 _obj,		\
                                  /*in*/   type a1,			\
				  /*out*/  type *a2,			\
				  /*inout*/type *a3,			\
					   CORBA_Environment *_ev) {	\
	*a2 = *a3;							\
	*a3 = a1;							\
	a1 = *a2;							\
	return a1;							\
}

void test1_test_void(test1 _obj, CORBA_Environment *_ev) {}

MKTST(short, CORBA_short);
MKTST(long, CORBA_long);
MKTST(ushort, test1_ushort);
MKTST(ulong, test1_ulong);
MKTST(float, CORBA_float);
MKTST(double, CORBA_double);
MKTST(boolean, CORBA_boolean);
MKTST(char, CORBA_char);
MKTST(octet, CORBA_octet);

#define flick_copy_const_string(dest,src) {			\
	dest = (char *)CORBA_alloc(sizeof(char)*strlen(src)+1);	\
	strcpy(dest, src);					\
}

CORBA_char *test1_test_string(test1 _obj,
			 /*in*/ CORBA_char *a1, /*out*/  CORBA_char **a2,
			      /*inout*/CORBA_char **a3, CORBA_Environment *_ev)
{
	*a2 = *a3;							
	flick_copy_const_string(*a3, a1);
	flick_copy_const_string(a1, "Groovy!");
	return a1;
}

CORBA_Object test1_test_Object(test1 _obj, CORBA_Object a1,
			       CORBA_Object *a2, CORBA_Object *a3, CORBA_Environment *_ev) 
{
	*a2 = CORBA_Object_duplicate(*a3);
	CORBA_Object_release(*a3);
	*a3 = CORBA_Object_duplicate(a1);
	CORBA_Object_release(a1);
	a1 = CORBA_Object_duplicate(*a2);
	CORBA_Object_release(*a2);
	*a2 = CORBA_Object_duplicate(*a3);
	return CORBA_Object_duplicate(a1);
}

void test1_test_throw(test1 _obj, CORBA_long case_num, CORBA_Environment *_ev) {}

void test1_please_exit(test1 _obj, CORBA_Environment *_ev) {
	printf("Server should shut down...\n");
        close(_obj->boa->socket_fd);
	exit(0);
}

/* End of file. */

