/*
 * Copyright (c) 1996, 1997 The University of Utah and
 * the Computer Systems Laboratory at the University of Utah (CSL).
 *
 * This file is part of Flick, the Flexible IDL Compiler Kit.
 *
 * Flick is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * Flick is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Flick; see the file COPYING.  If not, write to
 * the Free Software Foundation, 59 Temple Place #330, Boston, MA 02111, USA.
 */

#include <assert.h>
#include <stdio.h>
#include <stdlib.h>
#include "ttcp-client.h"
#include <sys/time.h>

/* Function to print statistics */
void print_stats(int size, struct timeval starttime, struct timeval endtime, int samples)
{
        double msecs = (1000.0 * (endtime.tv_sec - starttime.tv_sec)) +
                ( (endtime.tv_usec - starttime.tv_usec) / 1000.0);
	printf("%d\t%3.2f\n", size,
	       (double)(size * samples) * 8.0/(1048576.0 * msecs / 1000.0));
}

int main(int argc, char **argv)
{
	CORBA_Object obj;
	
	int qty = 65536, i, j;
        structBuffer sbuf;
        intBuffer ibuf;
	
	obj = CORBA_ORB_string_to_object(argv[1]);
	if (!obj) {
		printf("Problem in string to object conversion...\n");
		exit(1);
	}
	printf("Structures\nData\tMbps\n");
        for (i = 256; i <= 262144; i+= 64 ) {
                struct timeval starttime, endtime;
		CORBA_Environment _ev;
                sbuf._length = i;
		sbuf._buffer = (b *)calloc(i, sizeof(b));
		
		ttcp_structtest(obj, &sbuf, &_ev);
		qty = 0x400000 / (i * sizeof(b));
		if (qty > 1024)
			qty = 1024;
		gettimeofday(&starttime, NULL);
                for (j = 0; j < qty; j++) {
                        ttcp_structtest(obj, &sbuf, &_ev);
			switch (_ev._major) {
			case CORBA_NO_EXCEPTION:
				break;
			case CORBA_SYSTEM_EXCEPTION:
				fprintf(stderr, "ERROR %d.%d\n",
					((CORBA_system_exception *)_ev._except)->minor,
					((CORBA_system_exception *)_ev._except)->completed);
				CORBA_flick_free(_ev._except);
				exit(1);
				break;
			case CORBA_USER_EXCEPTION:
				if (_ev._atomic_user_id == 0) { /* the 'useless' exception */
					if (!(((useless *)(_ev._except))->baloney % 100000))
						fprintf(stderr, "Useless exception raised - %d\n",
							((useless *)(_ev._except))->baloney);
					CORBA_flick_free(_ev._except);
					break;
				}
			default:
				fprintf(stderr, "Unknown exception\n");
				exit(2);
			}
		}
		

                gettimeofday(&endtime, NULL);
                print_stats(i*16, starttime, endtime, qty);
		free(sbuf._buffer);
                qty /= 3;
        }
	
        qty = 65536;
	printf("Integers\nData\tMbps\n");
        for (i = 1024; i <= 1024 * 1024; i += 64) {
                struct timeval starttime, endtime;
		CORBA_Environment _ev;
		
		ibuf._length = i;
		ibuf._buffer = (int *)calloc(i, sizeof(int));
		
		ttcp_inttest(obj, &ibuf, &_ev);
		qty = 0x400000 / (i * sizeof(int));
		if (qty > 1024)
			qty = 1024;
		
                gettimeofday(&starttime, NULL);
                for (j = 0; j < qty; j++) {
                        ttcp_inttest(obj, &ibuf, &_ev);
			switch (_ev._major) {
			case CORBA_NO_EXCEPTION:
				break;
			case CORBA_SYSTEM_EXCEPTION:
				fprintf(stderr, "ERROR %d.%d\n",
					((CORBA_system_exception *)_ev._except)->minor,
					((CORBA_system_exception *)_ev._except)->completed);
				CORBA_flick_free(_ev._except);
				exit(1);
				break;
			case CORBA_USER_EXCEPTION:
				if (_ev._atomic_user_id == 0) { /* the 'useless' exception */
					if (!(((useless *)(_ev._except))->baloney % 100000))
						fprintf(stderr, "Useless exception raised - %d\n",
							((useless *)(_ev._except))->baloney);
					CORBA_flick_free(_ev._except);
					break;
				}
			default:
				fprintf(stderr, "Unknown exception\n");
				exit(2);
			}
		}
                gettimeofday(&endtime, NULL);
                print_stats(i * 4, starttime, endtime, qty);
		free(ibuf._buffer);
                qty /= 3;
        }
	return 0;
}

/* End of file. */

