/*
 * Copyright (c) 1995, 1996 The University of Utah and
 * the Computer Systems Laboratory at the University of Utah (CSL).
 *
 * This file is part of Flick, the Flexible IDL Compiler Kit.
 *
 * Flick is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * Flick is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Flick; see the file COPYING.  If not, write to
 * the Free Software Foundation, 59 Temple Place #330, Boston, MA 02111, USA.
 */

#if 0
#include <stdlib.h>
#else
#include <libc.h>
#endif
#include <stdio.h>
#include <unistd.h>
#include <errno.h>
#include <sys/types.h>
#include <sys/time.h>
#include <sysent.h>
#include <mach/message.h>
#include <mach/port.h>
#include <mach/mach_port.h>
#include <mach/mig_errors.h>
#include <mach/mach_traps.h>
#include <machine/endian.h>

/* These need to be success/failure functions
   that call the respective client/server stubs
   */

int call_server(mach_msg_header_t *request_ptr, mach_msg_header_t *reply_ptr);
int call_client(mach_port_t right);

#define MSGBUFSIZE (65*1024)

/* Number of times to bounce a message back and forth */
#define COUNT 1/*500000*/

#define SERVER_NAME 0xabadcafe
#define SERVER_RIGHT MACH_MSG_TYPE_COPY_SEND

/* The time array gets passes in timing messages and gets filled in
   at various points along the way.  */
typedef int time_array_t[32];
struct timeval tv = {1, 0};

#define RPC_IN 0
#define CALL_USER_STUB_ENTER 1
#define CALL_USER_STUB_SEND 2
#define CALL_KERN_TRAP_ENTER 3
#define CALL_KERN_TRAP_COPYIN_START 4
#define CALL_KERN_TRAP_COPYIN_END 5
#define CALL_KERN_TRAP_A 6
#define CALL_KERN_TRAP_B 7
#define CALL_KERN_TRAP_C 8
#define CALL_KERN_TRAP_D 9
#define CALL_KERN_TRAP_COPYOUT 10
#define CALL_KERN_TRAP_PUT 11
#define CALL_KERN_TRAP_LEAVE 12
#define CALL_SERVER_RECEIVE 13
#define CALL_SERVER_STUB_ENTER 14
#define CALL_SERVER_STUB_LEAVE 15
#define REPLY_SERVER_STUB_ENTER 16
#define REPLY_SERVER_STUB_LEAVE 17
#define REPLY_SERVER_SEND 18
/* Same as above */
#define REPLY_USER_STUB_RECEIVE 29
#define REPLY_USER_STUB_LEAVE 30
#define RPC_OUT 31

void doparent(int child_pid,
	      int child_thread,
	      mach_port_t right,
	      mach_port_t task_self);

void dochild(mach_port_t prent_right,
	     mach_port_t parent_task);

#if 0
/* Routine to read the 66MHz CPU timer */
static inline unsigned int
read_cpu_time (void)
{
	unsigned int time;
#if BYTE_ORDER == BIG_ENDIAN
	__asm __volatile ("mfctl %%cr16,%0" : "=r" (time));
#endif
	return time;
}
#endif

void
die(char *mes, int code)
{
	printf("fatal error: %s (%d, 0x%08x)\n", mes, code, code);
/*	printf("About to seg fault [weird - exit(1) causes a seg fault].\n");*/
	exit(1);
	printf("We haven't died!\n");
}

#define diemach(func, params) \
({ int err = func params; if (err) die (#func, err); })

__inline mach_msg_header_t *
dispatch(mach_msg_header_t *request_ptr, mach_msg_header_t *reply_ptr)
{
	if (!call_server(request_ptr, reply_ptr)) {
		printf("%08x %08x %d\n",
		       request_ptr->msgh_bits,
		       request_ptr->msgh_size,
		       request_ptr->msgh_id);
		printf("Invalid message (id = %d) not deallocated.\n",
		       request_ptr->msgh_id);
	}
	return reply_ptr;       
}

void *__buffer = 0;
int   __buflen = 0;

int
main(void)
{
	int child_pid;
	mach_port_t task_self = mach_task_self();
	mach_port_t right;

	/* Create a receive right to receive requests from the client */
	diemach(mach_port_allocate,(task_self, MACH_PORT_RIGHT_RECEIVE, &right));

	/* Fork into two processes */
	child_pid = fork ();
	if (child_pid == -1)
		die ("can't fork", errno);
	if (child_pid != 0) {
		/* parent */
		doparent(child_pid, 0, right, task_self);
	} else {
		/* child */
		dochild(right, task_self);
	}
	return 0;
}

typedef union {
	mach_msg_header_t	hdr;
	mig_reply_header_t	death_pill;
	char			space[MSGBUFSIZE];
} reply_message;

void
doparent(int child_pid, int child_thread, mach_port_t right, mach_port_t task_self)
{
	/*int i;*/
	reply_message msg_buffer_1, msg_buffer_2;

	mach_msg_header_t * request_ptr;
	mig_reply_header_t * reply_ptr;
	mach_msg_header_t * tmp;

	int ret;
	/*int act_sp;
	mach_port_t act;
	int dp;*/
      
	request_ptr = &msg_buffer_1.hdr;
	reply_ptr = &msg_buffer_2.death_pill;
	printf("server buffers at %08x-%08x and %08x-%08x\n",
	       (int)request_ptr, (int)request_ptr+MSGBUFSIZE,
	       (int)reply_ptr, (int)reply_ptr+MSGBUFSIZE);

  
	/* Give the child our task port so it can grab a send right to our receive port */
	printf("Inserting myself (0x%08x) into task 0x%08x with name 0x%08x, type = 0x%08x\n", task_self, task_by_pid(child_pid), SERVER_NAME, SERVER_RIGHT);
	diemach (mach_port_insert_right,(task_by_pid (child_pid),
					 SERVER_NAME, task_self,
					 SERVER_RIGHT));
	
	while (1) {
		/*int val = 1;*/
		ret = mach_msg(request_ptr, MACH_RCV_MSG,
			       0, sizeof msg_buffer_1,
			       right,
			       MACH_MSG_TIMEOUT_NONE, MACH_PORT_NULL);
		
		if (ret != MACH_MSG_SUCCESS)
			die("ux_server_loop: receive", ret);
		while (ret == MACH_MSG_SUCCESS) {
			mig_reply_header_t *reply2;
#if 0
			reply2 = reply_ptr;
			reply2->Head = *(mach_msg_header_t *)dispatch(request_ptr, &(reply2->Head));
#else
			reply2 = (mig_reply_header_t *) dispatch(request_ptr, (mach_msg_header_t *)reply_ptr);
#endif
			if (reply2->Head.msgh_remote_port == MACH_PORT_NULL) {
				/* no reply port, just get another request */
				break;
			}
			
			if (reply2->RetCode == MIG_NO_REPLY) {
				/* deallocate reply port right */
				(void) mach_port_deallocate(task_self,
							    reply2->Head.msgh_remote_port);
				break;
			}
			/* Send reply to prev request and receive another: */
#if 0
			print_message(reply2, reply2->Head.msgh_size);
#endif
			ret = mach_msg(&reply2->Head,
				       MACH_SEND_MSG|MACH_RCV_MSG,
				       reply2->Head.msgh_size,
				       sizeof msg_buffer_2,
				       right,
				       MACH_MSG_TIMEOUT_NONE,
				       MACH_PORT_NULL);
			if (ret != MACH_MSG_SUCCESS) {
				if (ret == MACH_SEND_INVALID_DEST) {
				/* deallocate reply port right */
				/* XXX should destroy entire reply msg */
					(void) mach_port_deallocate(task_self,
								    reply2->Head.msgh_remote_port);
				} else
					die ("ux_server_loop: rpc", ret);
			}
			tmp = request_ptr;
			request_ptr = (mach_msg_header_t *) reply2;
			if (reply_ptr != (mig_reply_header_t *) tmp)
				reply_ptr = (mig_reply_header_t *) tmp;
		}
	}
	printf ("\nparent terminating\n");
}


void
dochild(mach_port_t parent_right, mach_port_t parent_task)
{
  int i;
  int acq_type;
  kern_return_t ret;
  mach_port_t right;

  __buffer = (int *)malloc(32768);
  __buflen = 32768;
  
  /* wait for server to get up and running */
  sleep(1);
  
  /* Grab a send right to the "server" */
  printf("child extracting\n");

  i =0;
  do {
	  printf("Asking for 0x%08x from name 0x%08x, type = 0x%08x\n", parent_right, SERVER_NAME, MACH_MSG_TYPE_MAKE_SEND);
	  ret = mach_port_extract_right(SERVER_NAME,
					parent_right,
					MACH_MSG_TYPE_MAKE_SEND,
					&right,
					&acq_type);
	  switch(ret) {
	  case KERN_INVALID_NAME:
		  fprintf(stderr, "Invalid Name: 0x%08x did not denote a right\n", right);
		  break;
	  case KERN_INVALID_RIGHT:
		  fprintf(stderr, "Invalid Right: 0x%08x denoted an invalid right\n", right);
		  break;
	  case KERN_SUCCESS:
		  printf("Got right 0x%08x, type = 0x%08x\n", right, acq_type);
		  break;
	  default:
		  printf("Unknown condition: 0x%08x\n", ret);
	  }
	  i++;
  } while (ret != KERN_SUCCESS && i<500);

  printf("Beginning...\n");
  
  for (i = 0; i < COUNT; i++) {
    if (!(i&1023))
      printf("Working (%d%% complete)\n",i*100/COUNT);
    diemach(call_client, (right));
  }

  printf("\nChild Terminated.\n");
  printf("Killing Parent.\n");
  kill(getppid(), 11);
  sleep(1);
  kill(getppid(), 9);
}
