/*
 * Copyright (c) 1996, 1997 The University of Utah and
 * the Computer Systems Laboratory at the University of Utah (CSL).
 *
 * This file is part of Flick, the Flexible IDL Compiler Kit.
 *
 * Flick is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * Flick is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Flick; see the file COPYING.  If not, write to
 * the Free Software Foundation, 59 Temple Place #330, Boston, MA 02111, USA.
 */

#include <assert.h>
#include <stdio.h>
#include <stdlib.h>
#include "ttcp-client.h"
#include <flick/link/suntcp.h>
#include <sys/time.h>

/* Function to print statistics */
void print_stats(int size, double msecs, int samples)
{
	printf("BUFFER SIZE: %d\n", size);
	printf("\tNUM SAMPLES: %d\n", samples);
	printf("\tTOTAL TIME:  %f\n", msecs);
}

int main(int argc, char **argv)
{
	CLIENT tmp, *c;
	FLICK_SERVER_LOCATION s;
	int qty = 512, i, j;
        structBuffer sbuf;
        intBuffer ibuf;
	CORBA_Environment _ev;
	c = &tmp;
	
	if (argc != 2) {
		fprintf(stderr, "Usage: %s <host>\n", argv[0]);
		exit(1);
	}
	s.server_name = argv[1];
	s.prog_num = _PROG_ttcp;
	s.vers_num = _VERS_ttcp;
	create_client(c, s);
	printf("Beginning\n");
	
        for (i = 1024; i <= 32768; i += 256) {
		double curtime = 0.0;
		struct timeval start, stop;
		struct timezone tz;
		
                sbuf._length = i;
		sbuf._buffer = (b *)calloc(i, sizeof(b));
		
		gettimeofday(&start, &tz);
		
                for (j=0; j < qty; j++) {
                        ttcp_structtest(c, &sbuf, &_ev);
                        /* no error test (null result) */
		}
		
		gettimeofday(&stop, &tz);
		curtime = ((double)(stop.tv_sec - start.tv_sec))*1000.0 + ((double)(stop.tv_usec - start.tv_usec))/1000.0;
		
                print_stats(i*16, curtime, qty);
		free(sbuf._buffer);
        }
	
	printf("Beginning\n");
	
        for (i = 4096; i <= 131072; i += 1024) {
		double curtime = 0.0;
		struct timeval start, stop;
		struct timezone tz;
		
		ibuf._length = i;
		ibuf._buffer = (int *)calloc(i, sizeof(int));
		
		gettimeofday(&start, &tz);
		
                for (j = 0; j < qty; j++) {
			ttcp_inttest(c, &ibuf, &_ev);
                        /* no error test (null result) */
		}
		
		gettimeofday(&stop, &tz);
		curtime = ((double)(stop.tv_sec - start.tv_sec))*1000.0 + ((double)(stop.tv_usec - start.tv_usec))/1000.0;
		
                print_stats(i * 4, curtime, qty);
		free(ibuf._buffer);
        }
	
	return 0;
}

/* End of file. */

